@@ -338,7 +338,7 @@ class ApplicationSession(BaseSession):
         txaio.add_callbacks(
             d,
             lambda _: txaio.as_future(self.onConnect),
-            None,
+            lambda fail: self._swallow_error(fail, "While calling 'onConnect'")
         )
 
     @public
@@ -511,7 +511,6 @@ class ApplicationSession(BaseSession):
                 )
 
             elif isinstance(msg, message.Abort):
-
                 # fire callback and close the transport
                 details = types.CloseDetails(msg.reason, msg.message)
                 d = txaio.as_future(self.onLeave, details)
@@ -1163,7 +1162,9 @@ class ApplicationSession(BaseSession):
                 request_id=request.request_id,
                 request_type=request.__class__.__name__,
             )
-            txaio.reject(request.on_reply, exc)
+            if not txaio.is_called(request.on_reply):
+                txaio.reject(request.on_reply, exc)
+
             # wait for any async-ness in the error handlers for on_reply
             txaio.add_callbacks(d, lambda _: request.on_reply, lambda _: request.on_reply)
         return d
@@ -1266,6 +1267,16 @@ class ApplicationSession(BaseSession):
                                       args=args,
                                       kwargs=kwargs)
 
+        if options:
+            if options.correlation_id is not None:
+                msg.correlation_id = options.correlation_id
+            if options.correlation_uri is not None:
+                msg.correlation_uri = options.correlation_uri
+            if options.correlation_is_anchor is not None:
+                msg.correlation_is_anchor = options.correlation_is_anchor
+            if options.correlation_is_last is not None:
+                msg.correlation_is_last = options.correlation_is_last
+
         if options and options.acknowledge:
             # only acknowledged publications expect a reply ..
             on_reply = txaio.create_future()
@@ -1313,6 +1324,16 @@ class ApplicationSession(BaseSession):
             else:
                 msg = message.Subscribe(request_id, topic)
 
+            if options:
+                if options.correlation_id is not None:
+                    msg.correlation_id = options.correlation_id
+                if options.correlation_uri is not None:
+                    msg.correlation_uri = options.correlation_uri
+                if options.correlation_is_anchor is not None:
+                    msg.correlation_is_anchor = options.correlation_is_anchor
+                if options.correlation_is_last is not None:
+                    msg.correlation_is_last = options.correlation_is_last
+
             self._transport.send(msg)
             return on_reply
 
@@ -1428,6 +1449,16 @@ class ApplicationSession(BaseSession):
                                    args=args,
                                    kwargs=kwargs)
 
+        if options:
+            if options.correlation_id is not None:
+                msg.correlation_id = options.correlation_id
+            if options.correlation_uri is not None:
+                msg.correlation_uri = options.correlation_uri
+            if options.correlation_is_anchor is not None:
+                msg.correlation_is_anchor = options.correlation_is_anchor
+            if options.correlation_is_last is not None:
+                msg.correlation_is_last = options.correlation_is_last
+
         # FIXME: implement call canceling
         # def canceller(_d):
         #   cancel_msg = message.Cancel(request)
@@ -1480,6 +1511,16 @@ class ApplicationSession(BaseSession):
             else:
                 msg = message.Register(request_id, procedure)
 
+            if options:
+                if options.correlation_id is not None:
+                    msg.correlation_id = options.correlation_id
+                if options.correlation_uri is not None:
+                    msg.correlation_uri = options.correlation_uri
+                if options.correlation_is_anchor is not None:
+                    msg.correlation_is_anchor = options.correlation_is_anchor
+                if options.correlation_is_last is not None:
+                    msg.correlation_is_last = options.correlation_is_last
+
             self._transport.send(msg)
             return on_reply
 
@@ -1612,7 +1653,7 @@ class _SessionShim(ApplicationSession):
         # here we check that any duplicate keys have the same values
         authextra = authenticator.authextra
         merged = self._merged_authextra()
-        for k, v in merged:
+        for k, v in merged.items():
             if k in authextra and authextra[k] != v:
                 raise ValueError(
                     "Inconsistent authextra values for '{}': '{}' vs '{}'".format(
